/*  Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; version 2 of the
    License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */


CREATE DATABASE IF NOT EXISTS password_policy;

CREATE TABLE password_policy.password_policy_configuration (
 config_key VARCHAR(50) NOT NULL PRIMARY KEY,
 config_value INT NOT NULL DEFAULT 0
);

CREATE TABLE password_policy.password_policy_user_list (
 user CHAR(16) COLLATE utf8_bin NOT NULL DEFAULT '',
 host CHAR(60) COLLATE utf8_bin NOT NULL DEFAULT '',
 check_expiration ENUM('REPORT', 'EXPIRE', 'IGNORE') DEFAULT 'REPORT',
 check_no_password ENUM('REPORT', 'EXPIRE', 'IGNORE') DEFAULT 'REPORT',
 check_reused_password ENUM('REPORT', 'EXPIRE', 'IGNORE') DEFAULT 'REPORT', 
  PRIMARY KEY (user, host)
);

CREATE TABLE password_policy.user_account_history (
 id BIGINT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
 host CHAR(60) COLLATE utf8_bin NOT NULL DEFAULT '',
 user CHAR(16) COLLATE utf8_bin NOT NULL DEFAULT '',
 password CHAR(41) CHARACTER SET latin1 COLLATE latin1_bin NOT NULL DEFAULT '',
 created_date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
 KEY (host, user, created_date DESC)
);

CREATE TABLE password_policy.user_password_policy_violation (
 id BIGINT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
 history_id BIGINT UNSIGNED NOT NULL,
 host CHAR(60) COLLATE utf8_bin NOT NULL DEFAULT '',
 user CHAR(16) COLLATE utf8_bin NOT NULL DEFAULT '',
 violation_type INT NOT NULL,
 violation_resolved TINYINT NOT NULL DEFAULT 0,
 violation_message VARCHAR(255),
 violation_reported_date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
 KEY (history_id, violation_type, violation_resolved)
);


DELIMITER &&
CREATE DEFINER = pwd_policy@localhost
PROCEDURE password_policy.check_password_policy()
 BEGIN
 DECLARE pwd_expiration_days INT DEFAULT 90;
 DECLARE pwd_reuse_threshold INT DEFAULT 10;
 DECLARE done, new_id INT DEFAULT FALSE;
 DECLARE chk_pwd_expiration, chk_pwd_reuse, chk_no_pwd INT DEFAULT TRUE;
 DECLARE u_chk_pwd_expiration, u_chk_pwd_reuse, u_chk_no_pwd VARCHAR(10);
 DECLARE expired_user INT DEFAULT FALSE;
 DECLARE l_user, l_host, l_password VARCHAR(100);
 DECLARE cur CURSOR FOR 
   SELECT u.user, u.host, u.password, pul.check_expiration, pul.check_reused_password, pul.check_no_password 
   FROM mysql.user u
   JOIN password_policy.password_policy_user_list pul
   ON (u.user = pul.user AND u.host = pul.host)
   WHERE plugin = ''
   OR plugin = 'mysql_native_password';
 DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;
 
 
 -- populate the per-user configuration table, default behavior to report only.
 INSERT IGNORE INTO password_policy.password_policy_user_list (user, host)
   SELECT user, host FROM mysql.user WHERE plugin = ''
   OR plugin = 'mysql_native_password';

 SELECT config_value FROM password_policy_configuration WHERE config_key = 'password_expiration' INTO pwd_expiration_days;
 SELECT config_value FROM password_policy_configuration WHERE config_key = 'password_reuse_threshold' INTO pwd_reuse_threshold;
 SELECT config_value FROM password_policy_configuration WHERE config_key = 'check_password_expiration' INTO chk_pwd_expiration;
 SELECT config_value FROM password_policy_configuration WHERE config_key = 'check_password_reuse' INTO chk_pwd_reuse;
 SELECT config_value FROM password_policy_configuration WHERE config_key = 'check_no_password' INTO chk_no_pwd;

 -- Need to reset this, because HANDLER gets triggered if no row is found in any of the above
 SET done = FALSE;

 OPEN cur;

 read_loop: LOOP
  FETCH cur INTO l_user, l_host, l_password,u_chk_pwd_expiration, u_chk_pwd_reuse, u_chk_no_pwd;
  IF done THEN
    LEAVE read_loop;
  END IF;
  SET new_id = FALSE;
  CALL password_policy.record_password_history( l_user, l_host, l_password, new_id);
  IF new_id THEN
   -- Clear outstanding unresolved violation records for user/host
   UPDATE password_policy.user_password_policy_violation
     SET violation_resolved = 1
     WHERE user = l_user AND host = l_host AND violation_resolved = 0;
   IF l_password = '' THEN
    IF chk_no_pwd THEN
      -- reset password if configured:
      IF chk_no_pwd = 2 AND u_chk_no_pwd = 'EXPIRE' THEN
	CALL password_policy.force_password_reset(l_user, l_host);
	SET expired_user = TRUE;
      END IF;
      IF u_chk_no_pwd != 'IGNORE' THEN
        INSERT INTO password_policy.user_password_policy_violation
         (history_id, user, host, violation_type, violation_message, violation_resolved)
        VALUES
         (new_id, l_user, l_host, 1, CONCAT(l_user, '@', l_host, ' has no password.'), IF(expired_user, 1, 0));
      END IF;
     END IF;
     SET expired_user = FALSE;
   ELSE
    IF chk_pwd_reuse AND u_chk_pwd_reuse != 'IGNORE' THEN
      IF chk_pwd_reuse = 2 AND u_chk_pwd_reuse = 'EXPIRE' THEN
        CALL password_policy.check_password_reuse( new_id, l_user, l_host, l_password, pwd_reuse_threshold, 2 );
      ELSE
        CALL password_policy.check_password_reuse( new_id, l_user, l_host, l_password, pwd_reuse_threshold, 1 );
      END IF;
    END IF;
   END IF;
  ELSE
   IF chk_pwd_expiration AND u_chk_pwd_expiration != 'IGNORE' THEN
     IF chk_pwd_expiration = 2 AND u_chk_pwd_expiration = 'EXPIRE' THEN
       CALL password_policy.check_password_expiration( l_user, l_host, l_password, pwd_expiration_days, 2 );
     ELSE
       CALL password_policy.check_password_expiration( l_user, l_host, l_password, pwd_expiration_days, 1 );
     END IF;     
   END IF;
  END IF;
 END LOOP;

 CLOSE cur;
END
&& 

CREATE DEFINER = pwd_policy@localhost
PROCEDURE password_policy.check_password_expiration(
   IN l_user VARCHAR(60), 
   IN l_host VARCHAR(60),
   IN l_pass VARCHAR(41),
   IN days_threshold INT,
   IN expire_password INT)
 BEGIN
 DECLARE done INT DEFAULT FALSE;
 DECLARE l_password VARCHAR(41);
 DECLARE l_days INT;
 DECLARE new_id BIGINT UNSIGNED;
 DECLARE cur CURSOR FOR 
   SELECT h.id, h.password, DATEDIFF(CURRENT_DATE(), DATE(h.created_date))
   FROM password_policy.user_account_history h
   LEFT JOIN password_policy.user_password_policy_violation v ON 
    (h.id = v.history_id AND v.violation_type = 2 
     AND v.violation_reported_date
     AND v.violation_resolved = 0)
   WHERE h.user = l_user
   AND h.host = l_host
   AND v.id IS NULL
   ORDER BY h.created_date DESC LIMIT 1;
 DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

 OPEN cur;

 read_loop: LOOP
  FETCH cur INTO new_id, l_password, l_days;
  IF done THEN
    LEAVE read_loop;
  END IF;
  IF l_password = l_pass AND l_days >= days_threshold AND days_threshold > 0 THEN
   SET done = TRUE;

   -- expire the user account if configred to do so:	   
   IF expire_password = 2 THEN
     CALL password_policy.force_password_reset(l_user, l_host);
   END IF;

   -- record the password violation, mark as resolved if the password was expired.
   INSERT INTO password_policy.user_password_policy_violation
     (history_id, user, host, violation_type, violation_message, violation_resolved)
   VALUES
     (new_id, l_user, l_host, 2, CONCAT(l_user, '@', l_host, ' has not changed password in ', l_days, ' days.'), IF(expire_password = 2, 1, 0));
  END IF;
 END LOOP;

 CLOSE cur;
END
&& 

CREATE DEFINER = pwd_policy@localhost
PROCEDURE password_policy.check_password_reuse(
   IN new_id BIGINT UNSIGNED,
   IN l_user VARCHAR(60), 
   IN l_host VARCHAR(60),
   IN l_pass VARCHAR(41),
   IN reuse_check_threshold INT,
   IN expire_password INT)
 BEGIN
 DECLARE done INT DEFAULT FALSE;
 DECLARE l_password VARCHAR(41);
 DECLARE l_date DATE;
 DECLARE first_record INT DEFAULT TRUE;
 DECLARE cur CURSOR FOR 
   SELECT password, DATE(created_date)
   FROM password_policy.user_account_history
   WHERE user = l_user
   AND host = l_host
   ORDER BY created_date DESC;
 DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

 OPEN cur;

 read_loop: LOOP
  FETCH cur INTO l_password, l_date;
  IF done THEN
    LEAVE read_loop;
  END IF;
  IF reuse_check_threshold > 0 THEN
   SET reuse_check_threshold = reuse_check_threshold - 1;
  END IF;
  IF (NOT first_record) AND l_password = l_pass THEN
   SET done = TRUE;
   IF expire_password = 2 THEN
     CALL password_policy.force_password_reset(l_user, l_host);
   END IF;
   INSERT INTO password_policy.user_password_policy_violation
     (history_id, user, host, violation_type, violation_message, violation_resolved)
   VALUES
     (new_id, l_user, l_host, 3, CONCAT(l_user, '@', l_host, ' reused password from ', l_date), IF(expire_password = 2, 1, 0));
  END IF;
  SET first_record = FALSE;
  IF reuse_check_threshold = 0 THEN
   SET done = TRUE;
  END IF;
 END LOOP;

 CLOSE cur;
END
&& 

CREATE DEFINER = pwd_policy@localhost
PROCEDURE password_policy.record_password_history(
   IN l_user VARCHAR(60), 
   IN l_host VARCHAR(60),
   IN l_pass VARCHAR(41),
   INOUT new_id INT)
 BEGIN
 DECLARE done, found INT DEFAULT FALSE;
 DECLARE l_password VARCHAR(41);
 DECLARE cur CURSOR FOR 
   SELECT password 
   FROM password_policy.user_account_history
   WHERE user = l_user
   AND host = l_host
   ORDER BY created_date DESC LIMIT 1;
 DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

 OPEN cur;

 read_loop: LOOP
  FETCH cur INTO l_password;
  IF done THEN
    LEAVE read_loop;
  END IF;
  IF l_password = l_pass THEN
   SET found = TRUE;
  END IF;
 END LOOP;

 CLOSE cur;
 IF NOT found THEN
  INSERT INTO password_policy.user_account_history
   (user, host, password)
  VALUES
   (l_user, l_host, l_pass);
  SET new_id = LAST_INSERT_ID();
 END IF;
END
&& 


CREATE DEFINER = pwd_policy@localhost
PROCEDURE password_policy.force_password_reset(
    IN l_user VARCHAR(60),
    IN l_host VARCHAR(60))
  BEGIN
  SET @sql = CONCAT('ALTER USER \'', l_user, '\'@\'', l_host, '\' PASSWORD EXPIRE');
  PREPARE stmt FROM @sql;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END
&&

DELIMITER ;


CREATE DEFINER = root@localhost
EVENT password_policy.password_policy_check_event
ON SCHEDULE EVERY 1 DAY
DO 
 CALL password_policy.check_password_policy();



