/*  Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; version 2 of the
    License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */


DROP TEMPORARY TABLE IF EXISTS password_policy.test_suite;
CREATE TEMPORARY TABLE password_policy.test_suite (
 id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
 test_case VARCHAR(255) NOT NULL,
 test_status ENUM('FAIL', 'PASS', 'INCOMPLETE') NOT NULL DEFAULT 'INCOMPLETE',
 test_notes VARCHAR(255)
);

INSERT INTO password_policy.test_suite (test_case) VALUES ('Stored procedures installed correctly');
SET @id = LAST_INSERT_ID();

TRUNCATE password_policy.password_policy_user_list;
TRUNCATE password_policy.password_policy_configuration;
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;


CALL password_policy.check_password_policy();

UPDATE password_policy.test_suite SET test_status = 'PASS' WHERE id = @id;

INSERT INTO password_policy.test_suite (test_case) VALUES ('Accounts exist to test null passwords');
SET @id = LAST_INSERT_ID();

SELECT @no_pwd_count := COUNT(*) FROM password_policy.user_password_policy_violation WHERE violation_type = 1;

UPDATE password_policy.test_suite 
 SET test_status = IF(@no_pwd_count > 1, 'PASS', 'FAIL'),
 test_notes = CONCAT(@no_pwd_count, ' accounts with no password for testing.')
 WHERE id = @id;

INSERT INTO password_policy.test_suite (test_case) VALUES ('Accounts exist to test reused/expired passwords');
SET @id = LAST_INSERT_ID();

SELECT @no_pwd_count := COUNT(*) FROM password_policy.user_account_history WHERE password <> '';

UPDATE password_policy.test_suite 
 SET test_status = IF(@pwd_count > 1, 'PASS', 'FAIL'),
 test_notes = CONCAT(@pwd_count, ' accounts with no password for testing.')
 WHERE id = @id;

INSERT INTO password_policy.test_suite (test_case) VALUES ('Global configuration to skip checking for no password works');
SET @id = LAST_INSERT_ID();


-- confirm that no violations are reported for missing passwords, globally:
TRUNCATE password_policy.password_policy_user_list;
TRUNCATE password_policy.password_policy_configuration;
INSERT INTO password_policy.password_policy_configuration VALUES ('check_no_password', 0);
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;
CALL password_policy.check_password_policy();

SELECT @no_pwd_count := COUNT(*) FROM password_policy.user_password_policy_violation WHERE violation_type = 1;
UPDATE password_policy.test_suite 
 SET test_status = IF(@no_pwd_count > 0, 'FAIL', 'PASS'),
 test_notes = IF(@no_pwd_count > 0, CONCAT(@no_pwd_count, ' accounts flagged as no-password violations.'), NULL)
 WHERE id = @id;

INSERT INTO password_policy.test_suite (test_case) VALUES ('Per-account configuration to skip checking for no password works');
SET @id = LAST_INSERT_ID();

-- confirm that no violations are reported for missing passwords for specific accounts:
UPDATE password_policy.password_policy_configuration SET config_value = 1 WHERE config_key = 'check_no_password';
TRUNCATE password_policy.user_password_policy_violation;
TRUNCATE password_policy.user_account_history;
CALL password_policy.check_password_policy();
SELECT @user := user, @host := host FROM password_policy.user_password_policy_violation WHERE violation_type = 1 LIMIT 1;
UPDATE password_policy.password_policy_user_list SET check_no_password = 'IGNORE' WHERE user = @user AND host = @host;
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;
CALL password_policy.check_password_policy();
SELECT @no_pwd_count := COUNT(*) FROM password_policy.user_password_policy_violation
  WHERE violation_type = 1 AND user = @user AND host = @host;

UPDATE password_policy.test_suite 
 SET test_status = IF(@no_pwd_count > 0, 'FAIL', 'PASS'),
 test_notes = IF(@no_pwd_count > 0, CONCAT(@user, '@', @host, ' account flagged as no-password violation.'), NULL)
 WHERE id = @id;

SELECT @no_pwd_count := COUNT(*) FROM password_policy.user_password_policy_violation
  WHERE violation_type = 1 AND (user <> @user AND host <> @host);
  
INSERT INTO password_policy.test_suite (test_case, test_status, test_notes) 
  VALUES ('Per-account configuration to skip checking for no password does not affect other accounts', 
    IF(@no_pwd_count > 0, 'PASS', 'INCOMPLETE'),
    IF(@no_pwd_count > 0, CONCAT(@no_pwd_count, ' other accounts listed as violations.'), 'Check to ensure multiple accounts are configured with no password to validate test.'));

TRUNCATE password_policy.password_policy_user_list;
TRUNCATE password_policy.password_policy_configuration;
INSERT INTO password_policy.password_policy_configuration VALUES ('password_expiration', 1);
INSERT INTO password_policy.password_policy_configuration VALUES ('check_no_password', 0);
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;
CALL password_policy.check_password_policy();
SELECT @user := user, @host := host FROM password_policy.user_account_history LIMIT 1;
-- change the password history date so that it was a day ago
UPDATE password_policy.user_account_history SET created_date = NOW() - INTERVAL 1 DAY;
CALL password_policy.check_password_policy();
SELECT @violation_count := COUNT(*) FROM password_policy.user_password_policy_violation 
  WHERE user = @user AND host = @host AND violation_type = 2;
INSERT INTO password_policy.test_suite (test_case, test_status, test_notes) 
  VALUES ('Password expiration threshold configuration honored.', 
    IF(@violation_count > 0, 'PASS', 'FAIL'),
    IF(@violation_count > 0, NULL, CONCAT(@host, '%', @host, ' account was not flagged as violation.')));
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;
UPDATE password_policy.password_policy_user_list SET check_expiration = 'IGNORE' WHERE user = @user AND host = @host;
CALL password_policy.check_password_policy();
SELECT @violation_count := COUNT(*) FROM password_policy.user_password_policy_violation 
  WHERE user = @user AND host = @host AND violation_type = 2;
INSERT INTO password_policy.test_suite (test_case, test_status, test_notes) 
  VALUES ('Per-account configuration to ignore password expiration works.', 
    IF(@violation_count = 0, 'PASS', 'FAIL'),
    IF(@violation_count = 0, NULL, CONCAT(@user, '%', @host, ' account was not flagged as violation.')));

TRUNCATE password_policy.password_policy_user_list;
TRUNCATE password_policy.password_policy_configuration;
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;

CALL password_policy.check_password_policy();
INSERT INTO password_policy.user_account_history (user, host, password) 
  VALUES (@user, @host, PASSWORD('Testing'));
CALL password_policy.check_password_policy();
SELECT @violation_count := COUNT(*) FROM password_policy.user_password_policy_violation 
  WHERE user = @user AND host = @host AND violation_type = 3;

INSERT INTO password_policy.test_suite (test_case, test_status, test_notes) 
  VALUES ('Password reuse checking works', 
    IF(@violation_count > 0, 'PASS', 'FAIL'),
    IF(@violation_count > 0, NULL, CONCAT(@user, '%', @host, ' account was not flagged as violation.')));



SELECT * FROM password_policy.test_suite;



TRUNCATE password_policy.password_policy_user_list;
TRUNCATE password_policy.password_policy_configuration;
TRUNCATE password_policy.user_account_history;
TRUNCATE password_policy.user_password_policy_violation;


